// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.


#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    indexer = new Indexer();

    setupModel();
    setupViews();

connect(ui->actionLoad,SIGNAL(triggered()),this,SLOT(load()));
connect(ui->actionSave,SIGNAL(triggered()),this,SLOT(save()));
connect(ui->actionIndex_Directory,SIGNAL(triggered()),this,SLOT(createIndex()));
connect(ui->actionBack,SIGNAL(triggered()),this,SLOT(stepUpDir()));
connect(ui->actionOpen_File_Folder,SIGNAL(triggered()),this,SLOT(openFileFolder()));
connect(ui->actionQuit,SIGNAL(triggered()),this,SLOT(close()));
connect(ui->actionBundle_Small_Files,SIGNAL(changed()),this,SLOT(refreshChart()));
connect(ui->actionRefresh_Chart,SIGNAL(triggered()),this,SLOT(refreshChart()));
connect(indexer,SIGNAL(finished()),this,SLOT(indexingFinished()));
connect(&indexStatusUpdateTimer,SIGNAL(timeout()),this,SLOT(updateIndexingStatus()));
}

void MainWindow::refreshChart()
{
    makePie(fldId);
}


void MainWindow::stepUpDir()
{
    QString addr = fldId;
    int sdp = addr.lastIndexOf(':');
    addr.remove(sdp,addr.size() - sdp);
    makePie(addr);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::openFileFolder()
{
    QModelIndex index = pieChart->currentIndex();
    QString url(model->data(model->index(index.row(),0)).toString());
    QFileInfo file(url);

    if(!file.isDir())
    {
        file.setFile(indexer->getFolder(fldId).name,url);
    }

    if(!file.isDir() && !file.isFile()){file = QFileInfo(indexer->getFolder(fldId).name);}
    QDesktopServices::openUrl(QUrl::fromUserInput(file.absoluteFilePath()));
}





void MainWindow::setupModel()
{
    model = new QStandardItemModel(8, 3, this);
    model->setHeaderData(0, Qt::Horizontal, tr("Label"));
    model->setHeaderData(1, Qt::Horizontal, tr("Quantity"));
    model->setHeaderData(2, Qt::Horizontal, tr("id"));
}



void MainWindow::setupViews()
{
    pieChart = new PieView;
    pieChart->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    pieChart->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ui->horizontalLayout->addWidget(pieChart);


    pieChart->setModel(model);
    ui->listView->setModel(model);
    QItemSelectionModel *selectionModel = new QItemSelectionModel(model);
    pieChart->setSelectionModel(selectionModel);
    ui->listView->setSelectionModel(selectionModel);

    connect(pieChart,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(doubleClickedOnPie(QModelIndex)));
    connect(pieChart,SIGNAL(clicked(QModelIndex)),this,SLOT(clickedOnPie(QModelIndex)));
}

void MainWindow::doubleClickedOnPie(QModelIndex index)
{
  //  qDebug() << "doubble clickeded on pie";
    if(index.column() == 1)
    {
    QString addr = model->data(model->index(index.row(),2)).toString();
    if(addr.size() > 0){
    makePie(addr);
    }
}
}

void MainWindow::clickedOnPie(QModelIndex index)
{
    if(index.column() == 1)
    {
        QVariant msgFileName = model->data(model->index(index.row(),0)).toString();
        QVariant msgFileSize = model->data(model->index(index.row(),1)).toString();
        QString msgText = msgFileName.toString() + " - " + QString("%0 MB").arg(QVariant(msgFileSize).toLongLong() / 1048576);
        ui->statusBar->showMessage(msgText);
    }
}



void MainWindow::makePie(QString id)
{
fldId = id;

Folder folder = indexer->getFolder(id);

Files miscFiles;

model->removeRows(0, model->rowCount(QModelIndex()), QModelIndex());

for(int i = 0; i < folder.files.size();i++)
{
    if(ui->actionBundle_Small_Files->isChecked()){

      float percentage = (float)folder.files.at(i).size/(float)folder.size;
    if(folder.files.at(i).size < (folder.filesSize/folder.files.size()) * 1.7 && percentage < 0.05f){miscFiles.append(folder.files.at(i)); continue;}
}
                    model->insertRows(0, 1, QModelIndex());
                    model->setData(model->index(0, 0, QModelIndex()),folder.files.at(i).name);
                    model->setData(model->index(0, 1, QModelIndex()),folder.files.at(i).size);
                    model->setData(model->index(0, 0, QModelIndex()),QColor(qrand() % 256, qrand() % 256, qrand() % 256), Qt::DecorationRole);
 }

for(int i = 0; i < folder.folders.size();i++)
{
                    model->insertRows(0, 1, QModelIndex());
                    model->setData(model->index(0, 0, QModelIndex()),folder.folders.at(i).name);
                    model->setData(model->index(0, 1, QModelIndex()),folder.folders.at(i).size);
                    model->setData(model->index(0, 2, QModelIndex()),QString("%0:%1").arg(id).arg(i));
                    model->setData(model->index(0, 0, QModelIndex()),QColor(qrand() % 256, qrand() % 256, qrand() % 256), Qt::DecorationRole);
 }


//Add up misc files
if(miscFiles.size() > 0){
qint64 miscFilesSize = 0;
for(int i = 0; i < miscFiles.size(); i++){miscFilesSize += miscFiles.at(i).size;}

if(miscFiles.size() < 15) {
for(int i = 0; i < miscFiles.size();i++){
model->insertRows(0, 1, QModelIndex());
model->setData(model->index(0, 0, QModelIndex()),miscFiles.at(i).name);
model->setData(model->index(0, 1, QModelIndex()),miscFiles.at(i).size);
model->setData(model->index(0, 0, QModelIndex()),QColor(qrand() % 256, qrand() % 256, qrand() % 256), Qt::DecorationRole);
}
} else {
    QString averageSize = QString::number((((float)miscFilesSize/(float)miscFiles.size())/1048576),'f',2);

    model->insertRows(0, 1, QModelIndex());
    model->setData(model->index(0, 0, QModelIndex()),QString("%0 files averaging %1 MBs").arg(miscFiles.size()).arg(averageSize));
    model->setData(model->index(0, 1, QModelIndex()),miscFilesSize);
    model->setData(model->index(0, 0, QModelIndex()),QColor(qrand() % 256, qrand() % 256, qrand() % 256), Qt::DecorationRole);
}
}




model->sort(1,Qt::DescendingOrder);
}


void MainWindow::updateIndexingStatus()
{
    ui->statusBar->showMessage(QString("Indexing... %0 Files   %1 Folders").arg(indexer->totalFiles).arg(indexer->totalFolders));
}

void MainWindow::createIndex()
{
    QString dir = QFileDialog::getExistingDirectory(this, tr("Index Directory"),
                                                    QDir::homePath(),
                                                    QFileDialog::ShowDirsOnly
                                                    | QFileDialog::DontResolveSymlinks);
    if(QFileInfo(dir).isDir()){
    this->setCursor(Qt::BusyCursor);
    this->setEnabled(false);
    indexer->indexDir(dir);
    this->indexStatusUpdateTimer.start(20);

} else {qDebug() << "Error";}
}

void MainWindow::indexingFinished()
{
    this->indexStatusUpdateTimer.stop();
    this->setCursor(Qt::ArrowCursor);
    this->setEnabled(true);
    ui->statusBar->showMessage(QString("Done Indexing, Indexed %0 Files and %1 Folders").arg(indexer->totalFiles).arg(indexer->totalFolders));
    makePie("");
}

void MainWindow::save()
{

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Index File"),
                                                    QDir::home().absoluteFilePath("index.idx"),
                               tr("Index Files (*.idx)"));

    if(!fileName.contains(".idx")){fileName.append(".idx");}

    if(QFileInfo(fileName).exists()){QFile::remove(fileName);} //Delete the file if it exists

    QFile file(fileName);
    file.open(QIODevice::WriteOnly);
    QDataStream out(&file);

    save(out,indexer->idx_folder);

    file.close();
}

void MainWindow::load()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Index File"),
                                                    QDir::homePath(),
                                                    tr("Index Files (*.idx)"));

    if(QFileInfo(fileName).isFile())
    {

    QFile file(fileName);
    file.open(QIODevice::ReadOnly);
    QDataStream in(&file);

    indexer->idx_folder = load(in);

    file.close();
    makePie("");
}
}




void MainWindow::save(QDataStream &out, Folder folder)
{

out << folder.files.size();
out << folder.folders.size();
out << folder.name;
out << folder.size;
out << folder.foldersSize;
out << folder.filesSize;
out << folder.items;

for(int i = 0; i < folder.files.size();i++)
{
    out << folder.files.at(i).name;
    out << folder.files.at(i).size;
}

for(int i = 0; i < folder.folders.size();i++)
{
    save(out,folder.folders.at(i));
}

}

Folder MainWindow::load(QDataStream &in)
{
    Folder folder;
    int files;
    int folders;


    in >> files;
    in >> folders;
    in >> folder.name;
    in >> folder.size;
    in >> folder.foldersSize;
    in >> folder.filesSize;
    in >> folder.items;

for(int i = 0; i < files;i++)
{
    File file;
    in >> file.name;
    in >> file.size;
    folder.files.append(file);
}

for(int i = 0; i < folders;i++)
{
    folder.folders.append(load(in));
}

return folder;
}

